Type: Package
Package: cardelino
Title: Clone Identification from Single Cell Data
Version: 1.14.0
Authors@R: 
    c(person(given = "Jeffrey", 
             family = "Pullin",
             role = "aut",
             email = "jpullin@svi.edu.au"),
      person(given = "Yuanhua",
             family = "Huang",
             role = "aut",
             email = "yuanhua@hku.hk"),
      person(given = "Davis",
             family = "McCarthy",
             role = c("aut", "cre"),
             email = "dmccarthy@svi.edu.au"))
Description: Methods to infer clonal tree configuration for a
        population of cells using single-cell RNA-seq data (scRNA-seq),
        and possibly other data modalities. Methods are also provided
        to assign cells to inferred clones and explore differences in
        gene expression between clones. These methods can flexibly
        integrate information from imperfect clonal trees inferred
        based on bulk exome-seq data, and sparse variant alleles
        expressed in scRNA-seq data. A flexible beta-binomial error
        model that accounts for stochastic dropout events as well as
        systematic allelic imbalance is used.
License: GPL-3
URL: https://github.com/single-cell-genetics/cardelino
BugReports: https://github.com/single-cell-genetics/cardelino/issues
Depends: R (>= 4.2), stats
Imports: combinat, GenomeInfoDb, GenomicRanges, ggplot2, ggtree,
        Matrix, matrixStats, methods, pheatmap, snpStats, S4Vectors,
        utils, VariantAnnotation, vcfR
Suggests: BiocStyle, foreach, knitr, pcaMethods, rmarkdown, testthat,
        VGAM
Enhances: doMC
VignetteBuilder: knitr
biocViews: SingleCell, RNASeq, Visualization, Transcriptomics,
        GeneExpression, Sequencing, Software, ExomeSeq
Encoding: UTF-8
NeedsCompilation: yes
RoxygenNote: 7.2.1
Config/pak/sysreqs: libcairo2-dev cmake libfontconfig1-dev
        libfreetype6-dev make libbz2-dev libicu-dev liblzma-dev
        libpng-dev libuv1-dev libxml2-dev libssl-dev xz-utils
        zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:59:16 UTC
RemoteUrl: https://github.com/bioc/cardelino
RemoteRef: RELEASE_3_23
RemoteSha: 65677bd3d8e5cc98920a03b4da08452ef614e995
Packaged: 2026-04-29 18:39:08 UTC; root
Author: Jeffrey Pullin [aut],
  Yuanhua Huang [aut],
  Davis McCarthy [aut, cre]
Maintainer: Davis McCarthy <dmccarthy@svi.edu.au>
Built: R 4.6.0; ; 2026-04-29 20:24:18 UTC; unix
