# Generated by roxygen2: do not edit by hand

export(assign_cells_to_clones)
export(assign_scores)
export(binaryPRC)
export(binaryROC)
export(clone_id)
export(clone_id_EM)
export(clone_id_Gibbs)
export(colMatch)
export(donor_read_simulator)
export(get_snp_matrices)
export(get_tree)
export(heat_matrix)
export(load_GT_vcf)
export(load_cellSNP_vcf)
export(mixBinom)
export(mtx_to_df)
export(multiPRC)
export(plot_config_diffs)
export(plot_tree)
export(predMixBinom)
export(prob_heatmap)
export(pub.theme)
export(read_vcf)
export(rowArgmax)
export(rowMax)
export(sample_seq_depth)
export(sample_tree_SNV)
export(sim_read_count)
export(vc_heatmap)
import(ggplot2)
import(matrixStats)
import(pheatmap)
import(snpStats)
import(stats)
import(utils)
importFrom(GenomeInfoDb,seqlengths)
importFrom(GenomeInfoDb,seqlevelsStyle)
importFrom(GenomicRanges,findOverlaps)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,subjectHits)
importFrom(VariantAnnotation,alt)
importFrom(VariantAnnotation,geno)
importFrom(VariantAnnotation,genotypeToSnpMatrix)
importFrom(VariantAnnotation,isSNV)
importFrom(VariantAnnotation,readVcf)
importFrom(VariantAnnotation,ref)
importFrom(combinat,permn)
importFrom(ggtree,geom_tiplab)
importFrom(ggtree,ggtree)
importFrom(methods,as)
importFrom(methods,is)
