# inst

This folder contains the files:

-   `CITATION`: Citation information for cardelino.
-   `cardelino_image_attribution.txt`: Attribution information for the photo for a goldfinch used in the package sticker.
-   `cardelino_sticker.png`: The cardelino's hex sticker.
-   `cardelino_med.jpg`: The photo of a goldfinch used in cardelino's sticker.

It also contains the folder `extdata`, which contains extra data used in the package's vignettes and examples.

Specifically, `extdata` contains the files:

-   `canopy_results.coveraged.rds`: Canopy tree results. Used in `vignette-cloneid.Rmd`.
-   `cells.donorid.vcf.gz`: A VCF data file from cells. Used in the examples of functions in `read_data.R`.
-   `cellSNP.cells.vcf.gz`: A VCF data file created using cellSNP. Used in the examples of functions in `read_data.R`.
-   `donors.donorid.vcf.gz`: A VCF data file from a donor. Used the in the examples of `get_snp_matrices()`.
-   `passed_ad.mtx`: AD matrix created using MQUAD. Used in the "Clone ID on mitochondrial variations" section of `vignette-cloneid.Rmd`.
-   `passed_dp.mtx`: DP matrix created using MQUAD. Used in the "Clone ID on mitochondrial variations" section of `vignette-cloneid.Rmd`.
-   `passed_variant_names.txt`: Variant names for the AD and DP matrices. Used in the "Clone ID on mitochondrial variations" section of `vignette-cloneid.Rmd`.
